//
//  UpnpAvRenderingControlService.m
//  uPnp2
//
//  Created by rk on 26/01/11.
//  Copyright 2011 Interchart. All rights reserved.
//

#import "UpnpAvRenderingControlService.h"


@implementation UpnpAvRenderingControlService


#pragma mark Designated Initializer
-(id)initWithService:(UpnpAvService *)theService {
	self = [super init];
	service = theService;
	desiredVolume = -1;
	return self;
}

#pragma mark AsynchUpnpCommandDelegate methods
- (void)handleAsynchUpnpCommandResult:(XMLTreeNode *)parsedResult {
	if ([UpnpAvRenderingControlService isSetVolumeResponse:parsedResult] &&
		volumeChangeIsPending) {
		[self asynchGetVolume:self];
	}
	
	if ([UpnpAvRenderingControlService isGetVolumeResponse:parsedResult]) {
		int currentVolume = [UpnpAvRenderingControlService getVolumeResponseValue:parsedResult];
		if (currentVolume == desiredVolume) {
			volumeChangeIsPending = false; // this method is only called if volume change called
			[desiredVolumeResponseHandler handleAsynchUpnpCommandResult:parsedResult];
			[self asynchGetMute:desiredVolumeResponseHandler];
			
		} else {
			[self sendSetVolumeCommand:desiredVolume];
		}
	}
}


#pragma mark -

- (void) asynchGetMute: (<AsynchUpnpCommandDelegate>)responseHandler {
	NSLog(@"%@: asynchGetMute", self);
	
	NSString *commandName = @"GetMute";
	NSMutableDictionary *args = [[NSMutableDictionary alloc] init];
	[args setValue:@"0" forKey:@"InstanceID"];
	[args setValue:@"Master" forKey:@"Channel"];
	
	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = responseHandler;
		
	[service executeCommandAsync:command];
}

- (void)asynchSetMute:(BOOL)newValue : (<AsynchUpnpCommandDelegate>)responseHandler {
	NSLog(@"%@: asynchSetMute", self);
	
	NSString *setValue = @"0";
	if (newValue) setValue = @"1";
	NSString *commandName = @"SetMute";
	NSMutableDictionary *args = [[NSMutableDictionary alloc] init];
	[args setValue:@"0" forKey:@"InstanceID"];
	[args setValue:@"Master" forKey:@"Channel"];
	[args setValue:setValue forKey:@"DesiredMute"];
	
	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = responseHandler;
	
	[service executeCommandAsync:command];
}

- (void)asynchGetVolume:(<AsynchUpnpCommandDelegate>)responseHandler {
	NSLog(@"%@: asynchGetVolume", self);
	
	NSString *commandName = @"GetVolume";
	NSMutableDictionary *args = [[NSMutableDictionary alloc] init];
	[args setValue:@"0" forKey:@"InstanceID"];
	[args setValue:@"Master" forKey:@"Channel"];
	
	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = responseHandler;
	
	[service executeCommandAsync:command];
}

- (void)asynchSetVolume:(int)newValue : (<AsynchUpnpCommandDelegate>)responseHandler {
	NSLog(@"%@: asynchSetVolume", self);
	desiredVolume = newValue;
	desiredVolumeResponseHandler = responseHandler;
	
	if (!volumeChangeIsPending) {
		volumeChangeIsPending = true;
		[self sendSetVolumeCommand:desiredVolume];
	}
	
}
#pragma mark Private methods

- (void)sendSetVolumeCommand:(int)newValue {
	NSLog(@"%@: sendSetVolumeCommand", self);
	NSString *setValue = [NSString stringWithFormat:@"%d", desiredVolume];
	NSString *commandName = @"SetVolume";
	NSMutableDictionary *args = [[NSMutableDictionary alloc] init];
	[args setValue:@"0" forKey:@"InstanceID"];
	[args setValue:@"Master" forKey:@"Channel"];
	[args setValue:setValue forKey:@"DesiredVolume"];
	
	AsynchUpnpCommand *command = [[AsynchUpnpCommand alloc] init];
	command.command = commandName;
	command.params = args;
	command.delegate = self;
	
	[service executeCommandAsync:command];
	
	
}

#pragma mark Static Utility Methods
+(bool)isGetMuteResponse:(XMLTreeNode *)parsedResult {
	if ([parsedResult findChild: @"s:Envelope/s:Body/u:GetMuteResponse/CurrentMute"]) {
		return true;
	} else {
		return false;
	}
}
+(bool)isSetMuteResponse:(XMLTreeNode *)parsedResult {
	if ([parsedResult findChild: @"s:Envelope/s:Body/u:SetMuteResponse"]) {
		return true;
	} else {
		return false;
	}
}
+(bool)isGetVolumeResponse:(XMLTreeNode *)parsedResult {
	if ([parsedResult findChild: @"s:Envelope/s:Body/u:GetVolumeResponse/CurrentVolume"]) {
		return true;
	} else {
		return false;
	}
}
+(bool)isSetVolumeResponse:(XMLTreeNode *)parsedResult {
	if ([parsedResult findChild: @"s:Envelope/s:Body/u:SetVolumeResponse"]) {
		return true;
	} else {
		return false;
	}
}

+(bool)getMuteResponseValue:(XMLTreeNode *)parsedResult {
	bool result = false;
	XMLTreeNode *currentMuteNode = [parsedResult findChild: @"s:Envelope/s:Body/u:GetMuteResponse/CurrentMute"];
	if (currentMuteNode) {
		int iCurrentMute = [currentMuteNode.text intValue];
		switch (iCurrentMute) {
			case 0:
				result = false;
				break;
			case 1:
				result = true;
				break;
			default:
				break;
		}
	}
	return result;
}

// returns -1 if the input is not a response from GetVolume command
+(int)getVolumeResponseValue:(XMLTreeNode *)parsedResult {
	int result = -1;
	XMLTreeNode *currentVolumeNode = [parsedResult findChild: @"s:Envelope/s:Body/u:GetVolumeResponse/CurrentVolume"];
	if (currentVolumeNode) {
		int iCurrentVolume = [currentVolumeNode.text intValue];
		result = iCurrentVolume;
	}
	return result;
}



@end
